/*
  gpsindicator.cpp
  Copyright (C) 2008-2009 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

#include "qgmap.h"
#include "gpsindicator.h"
#include <qframe.h>
#include <qlayout.h>
#include <qfont.h>
#include <qhbox.h>
#include <qlabel.h>
#include <qimage.h>

GPSIndicator::GPSIndicator(QWidget *parent, const char *name, WFlags f) :
	QFrame(parent, name, f)
{
#ifdef QWS
	QFont small_font("lcfont", 8);
	setFont(QFont("lcfont", 12));
#else
	QFont small_font("gothic", 6);
	setFont(QFont("gothic", 8));
#endif

	QVBoxLayout *vbox = new QVBoxLayout(this, 2);
	setFrameStyle(QFrame::Box | QFrame::Plain);

	QHBox *hbox = new QHBox(this);
	hbox->setSpacing(3);

	sate_icon = new QLabel(hbox);
	sate_icon->setPixmap(QPixmap(APP_PATH "/satellite.png"));
	satellites = new QLabel(":--",hbox);
	time = new QLabel("--:--:--",hbox);
	hbox->setStretchFactor(time, 1);
	lock_icon = new QLabel(hbox);
	lock_icon->setPixmap(QPixmap(APP_PATH "/lock.png"));

	vbox->addWidget(hbox);

	lat = new QLabel("Lat:  --.------",this);
	lon = new QLabel("Lon: ---.------",this);
	vbox->addWidget(lat);
	vbox->addWidget(lon);

	QHBox *hbox2 = new QHBox(this);
	hbox2->setSpacing(5);
	vel = new QLabel("---.-km/h", hbox2);
	vel->setFont(small_font);
	deg = new QLabel(QString::fromUtf8("---°"), hbox2);
	deg->setFont(small_font);
	alt = new QLabel("---m", hbox2);
	alt->setFont(small_font);
	vbox->addWidget(hbox2);

	disableText(true, true);
	adjustSize();
	setGeometry(30, 100, width(), height());

	interval = 3;
	tid = -1;
}

void GPSIndicator::show()
{
	QFrame::show();
	updateTimer();

	satellites->setText(":--");
	time->setText("--:--:--");
	lat->setText("Lat:  --.------");
	lon->setText("Lon: ---.------");
	vel->setText("---.-km/h");
	deg->setText(QString::fromUtf8("---°"));
	alt->setText("---m");
	lock_icon->show();

	disableText(true, true);

//	adjustSize();
	QWidget::show();
}

void GPSIndicator::hide()
{
	killTimers();
	tid = -1;
	QFrame::hide();
}

void GPSIndicator::updateInfo(GPSInfo &info)
{
	QString tmp;

	if(info.isUpdated()){
		if(info.isValid()){
			tmp.sprintf("Lat: %10.6f", info.getLatitude());
			lat->setText(tmp);
			tmp.sprintf("Lon:%11.6f", info.getLongitude());
			lon->setText(tmp);
			tmp.sprintf("%5.1fkm/h", info.getVelocity());
			vel->setText(tmp);
			tmp.sprintf("%3d", info.getDegree());
			tmp += QString::fromUtf8("°");
			deg->setText(tmp);
			tmp.sprintf("%3dm", (int)info.getAltitude());
			alt->setText(tmp);
		}
		tmp.sprintf(":%-2d", info.getSatellites());
		satellites->setText(tmp);
		tmp = info.getDateTime().time().toString();
		time->setText(tmp);

		updateTimer();
	}

	disableText(!info.isUpdated(), !info.isValid());
}

void GPSIndicator::timerEvent(QTimerEvent *e)
{
	killTimer(e->timerId());
	tid = -1;

	disableText(true, true);
	repaint();
}

void GPSIndicator::setInterval(int sec)
{
	interval = sec;
	updateTimer();
}

void GPSIndicator::disableText(bool info, bool loc)
{
	satellites->setDisabled(info);
	time->setDisabled(info);
	lat->setDisabled(loc);
	lon->setDisabled(loc);
	vel->setDisabled(loc);
	deg->setDisabled(loc);
	alt->setDisabled(loc);
}

void GPSIndicator::setLock(bool state)
{
	if(state)
		lock_icon->show();
	else
		lock_icon->hide();
}

bool GPSIndicator::isLock()
{
	return lock_icon->isVisible();
}

void GPSIndicator::updateTimer()
{
	if(tid >= 0){
		killTimer(tid);
	}
	tid = startTimer(interval * 2 * 1000);
}
