/*
  mappath.cpp
  Copyright (C) 2008-2009 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

//#define DEBUG

#include <stdio.h>
#include "qgmap.h"
#include "pos.h"
#include "mappath.h"
#include "mainwindow.h"
#include <qstring.h>
#include <qregexp.h>
#include <qfileinfo.h>
#include "mounter.h"

//#define pthread_mutex_lock(...) {printf("pthread_mutex_lock:%d\n", __LINE__);pthread_mutex_lock(__VA_ARGS__);}
//#define pthread_mutex_unlock(...) {printf("pthread_mutex_unlock:%d\n", __LINE__);pthread_mutex_unlock(__VA_ARGS__);}

MapPath::MapPath(QObject *parent)
	: QObject(parent)
{
	Enter();

	mounter = (Mounter *)lookupObject(parent, "Mounter", "mounter");

	/* connect signals */
	connect(parent, SIGNAL(configChanged(const QString &, const QString &)),
			this, SLOT(setConfig(const QString &, const QString &)));

	pthread_mutex_init(&lock, NULL);
}

MapPath::~MapPath()
{
	if(mounter)
		mounter->unmountAll();
}

QString MapPath::findFile(QString file)
{
	int i;
//	Enter();

	pthread_mutex_lock(&lock);

	for(i = 0; i < MAXMAPPATH; i++){
		if(base_path[i] == "")
			continue;

		QFileInfo info(base_path[i] + file);
//		printf("file:%s isFile=%d\n",
//			   info.filePath().ascii(), info.isFile());
		if (info.isFile()) {
			pthread_mutex_unlock(&lock);
			return info.filePath();
		}
	}

	pthread_mutex_unlock(&lock);
	return "";
}

QString MapPath::getPath(Pos &pos)
{
	unsigned int x,y;
	QString strX, strY;
	QString path;
	QString relpath;
	int i;
//	Enter();

	x = pos.getX() / 256;
	y = pos.getY() / 256;

	relpath.sprintf("/%02d", pos.getZoom());
	strX.sprintf("%010d", x);
	strY.sprintf("%010d", y);

	for(i=0;i<10;i++){
		relpath += '/';
		relpath += strX.mid(i, 1);
		relpath += strY.mid(i, 1);
	}

	relpath += ".png";

	path = MapPath::findFile(relpath);

	return path;
}

void MapPath::setPath(int index, const QString &value)
{
	int err;
	QString mnt;

	pthread_mutex_lock(&lock);

//	printf("index:%d\n", index);
	if(mnt_path[index] != ""){
		mounter->unmount(mnt_path[index]);
	}
	mnt_path[index] = "";
	base_path[index] = "";

	if(value == "") {
		pthread_mutex_unlock(&lock);
		return;
	}

	if(value.right(1) == "/"){
		//DIR
		QFileInfo dir(value);

		//実行時に存在しなければ登録しない
		if(dir.exists() && dir.isDir())
			base_path[index] = value;
//		printf("dir\n");
		pthread_mutex_unlock(&lock);
		return;
	}

	/* rom, iso のみ許可 */
	if(value.right(4).lower() != ".rom" &&
	   value.right(4).lower() != ".iso"){
//		printf("unknown ext\n");
		pthread_mutex_unlock(&lock);
		return;
	}


	mnt_path[index] = mounter->mount(value, &err);
//	printf("err=%d img=%s mnt=%s\n", err,
//		   value.ascii(), mnt_path[index].ascii());
	if(err){
		base_path[index] = "";
	}else{
		base_path[index] = mnt_path[index] + "/imgs/";
	}
	pthread_mutex_unlock(&lock);

//	printf("mnt_path = %s base_path = %s\n",
//		   mnt_path[index].ascii(), base_path[index].ascii());
}


void MapPath::setConfig(const QString &key, const QString &value)
{
	QRegExp regexp("^MapPath\\d$");

	Enter();

	if(!mounter)
		return;

//	printf("key:%s value:%s\n", key.ascii(), value.ascii());
	if(key.find(regexp) >= 0){
		int index = key.right(1).toInt();

		setPath(index, value);
	}
}
