/*
  slider.cpp
  Copyright (C) 2008-2009 Tomohiro Masubuchi <bucchi@e-ml.jp>

  This file is part of qgmap.

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; If not, see <http://www.gnu.org/licenses/>.
*/

#include <qwidget.h>
#include <qpixmap.h>
#include <qstring.h>
#include <qpainter.h>
#include "qgmap.h"
#include "slider.h"
#include "pos.h"
#include "layeredbase.h"

Slider::Slider( LayeredBase *parent, const char *name) :
	LayeredWidget(parent, name),
	slider_base(8,26), nob_center(9,5) , slider_mul(0,8)
{
	is_button = FALSE;
	value = 19;

	slider.load(SLIDERIMG_FILE);
	nob.load(NOBIMG_FILE);

	/* connect signal */
	connect(this, SIGNAL(valueChanged(int)),
			parent, SIGNAL(zoomChanged(int)));
	connect(parent, SIGNAL(positionChanged(Pos &)),
			this, SLOT(setPosition(Pos &)));
}

bool Slider::paintEvent(QPaintEvent *e, QPixmap &scr)
{
	UNUSED(e);

	Enter();

	QPainter p(&scr);

	QPoint position = pos();

	p.drawPixmap( position , slider );

	position += slider_base - nob_center;
	position += slider_mul * value;
	p.drawPixmap( position, nob );

	return true;
}

bool Slider::mousePressEvent(QMouseEvent *e)
{
	Enter();

	int z;
	int y = (e->pos() - pos()).y();

	if(y <= 16){
		updateValue(value-1);
		is_button = TRUE;
		return true;
	}
	if(y >= 188){
		updateValue(value+1);
		is_button = TRUE;
		return true;
	}

	z = y - slider_base.y() + slider_mul.y()/2;
	z /= slider_mul.y();

	if(updateValue(z))
		return true;

	return false;
}

bool Slider::mouseReleaseEvent(QMouseEvent *e)
{
	UNUSED(e);

	Enter();
	is_button = false;
	return false;
}

bool Slider::mouseDoubleClickEvent(QMouseEvent *e)
{
	UNUSED(e);
	return true;
}

bool Slider::mouseMoveEvent(QMouseEvent *e)
{
	Enter();
	if(!is_button)
		return mousePressEvent(e);
	return false;
}

void Slider::showEvent()
{
	Enter();
	QWidget *parent = parentWidget();
	int left_margin = 0;

	if (g_machine.type() != SL_A300)
		left_margin = 20;

	setGeometry(parent->width() - slider.width() - left_margin,
				(parent->height() - slider.height()) / 2,
				slider.width(), slider.height());
}

void Slider::setPosition(Pos &pos)
{
	updateValue(pos.getZoom());
}

bool Slider::updateValue( int value )
{
	Enter();

	if(value < 0 || value > 19
	   || this->value == value){
		return false;
	}

	this->value = value;

	emit valueChanged(value);
	repaint();

	return true;
}
